@@ -158,6 +158,11 @@ class AsyncClient(client.Client):
         """Create an event object."""
         return asyncio.Event()
 
+    def _reset(self):
+        if self.http:  # pragma: no cover
+            asyncio.ensure_future(self.http.close())
+        super()._reset()
+
     async def _connect_polling(self, url, headers, engineio_path):
         """Establish a long-polling connection to the Engine.IO server."""
         if aiohttp is None:  # pragma: no cover
@@ -229,6 +234,16 @@ class AsyncClient(client.Client):
             self.base_url = websocket_url
             self.logger.info(
                 'Attempting WebSocket connection to ' + websocket_url)
+
+        # get the cookies from the long-polling connection so that they can
+        # also be sent the the WebSocket route
+        cookies = None
+        if self.http:
+            cookies = '; '.join(["{}={}".format(cookie.key, cookie.value)
+                                 for cookie in self.http._cookie_jar])
+            headers = headers.copy()
+            headers['Cookie'] = cookies
+
         try:
             ws = await websockets.connect(
                 websocket_url + self._get_url_timestamp(),
@@ -242,21 +257,46 @@ class AsyncClient(client.Client):
             else:
                 raise exceptions.ConnectionError('Connection error')
         if upgrade:
-            await ws.send(packet.Packet(packet.PING, data='probe').encode(
-                always_bytes=False))
-            pkt = packet.Packet(encoded_packet=await ws.recv())
+            p = packet.Packet(packet.PING, data='probe').encode(
+                always_bytes=False)
+            try:
+                await ws.send(p)
+            except Exception as e:  # pragma: no cover
+                self.logger.warning(
+                    'WebSocket upgrade failed: unexpected send exception: %s',
+                    str(e))
+                return False
+            try:
+                p = await ws.recv()
+            except Exception as e:  # pragma: no cover
+                self.logger.warning(
+                    'WebSocket upgrade failed: unexpected recv exception: %s',
+                    str(e))
+                return False
+            pkt = packet.Packet(encoded_packet=p)
             if pkt.packet_type != packet.PONG or pkt.data != 'probe':
                 self.logger.warning(
                     'WebSocket upgrade failed: no PONG packet')
                 return False
-            await ws.send(packet.Packet(packet.UPGRADE).encode(
-                always_bytes=False))
+            p = packet.Packet(packet.UPGRADE).encode(always_bytes=False)
+            try:
+                await ws.send(p)
+            except Exception as e:  # pragma: no cover
+                self.logger.warning(
+                    'WebSocket upgrade failed: unexpected send exception: %s',
+                    str(e))
+                return False
             self.current_transport = 'websocket'
             if self.http:  # pragma: no cover
                 await self.http.close()
             self.logger.info('WebSocket upgrade was successful')
         else:
-            open_packet = packet.Packet(encoded_packet=await ws.recv())
+            try:
+                p = await ws.recv()
+            except Exception as e:  # pragma: no cover
+                raise exceptions.ConnectionError(
+                    'Unexpected recv exception: ' + str(e))
+            open_packet = packet.Packet(encoded_packet=p)
             if open_packet.packet_type != packet.OPEN:
                 raise exceptions.ConnectionError('no OPEN packet')
             self.logger.info(
@@ -307,7 +347,7 @@ class AsyncClient(client.Client):
 
     async def _send_request(
             self, method, url, headers=None, body=None):  # pragma: no cover
-        if self.http is None:
+        if self.http is None or self.http.closed:
             self.http = aiohttp.ClientSession()
         method = getattr(self.http, method.lower())
         try:
@@ -495,7 +535,6 @@ class AsyncClient(client.Client):
                 if r is None:
                     self.logger.warning(
                         'Connection refused by the server, aborting')
-                    self._reset()
                     break
                 if r.status != 200:
                     self.logger.warning('Unexpected status code %s in server '
@@ -512,6 +551,5 @@ class AsyncClient(client.Client):
                     self.logger.warning(
                         'Write loop: WebSocket connection was closed, '
                         'aborting')
-                    self._reset()
                     break
         self.logger.info('Exiting write loop task')
